"use strict";
class Cache extends WeakMap {
}
function stringify(obj, replacer) {
    let decycled = decycle(obj, replacer);
    return JSON.stringify(decycled);
}
exports.stringify = stringify;
/** Replace property values which are circular object references with its string representation */
function decycle(obj, replacer) {
    "use strict";
    if (replacer && !(replacer instanceof Function))
        throw Error("Invalid argument 'replacer', it should be a function or null.");
    let cache = new Cache();
    //return internalDecycle(cache, obj, "$", replacer);
    function internalDecycle(value, path) {
        if (typeof value === "object" && value !== null && isPrimitiveWrapper(value) === false) {
            if (replacer)
                replacer(value);
            let cachedPath = cache.get(value);
            if (cachedPath !== undefined) {
                return { $ref: cachedPath };
            }
            //Caches obj and it's path
            cache.set(value, path);
            if (Array.isArray(value)) {
                //  decycles each element
                let decycled = [];
                value.forEach(function (element, index) {
                    decycled[index] = internalDecycle(element, `${path}[${index}]`);
                });
                return decycled;
            }
            else {
                //  decycles each property value
                let decycled = {};
                Object.keys(value).forEach(function (name) {
                    decycled[name] = internalDecycle(value[name], `${path}[${name}]`);
                });
                return decycled;
            }
        }
        return value;
    }
    let result = internalDecycle(obj, "$");
    return result;
}
exports.decycle = decycle;
function parse(jsonText) {
    let jsonObject = JSON.parse(jsonText);
    return retrocycle(jsonObject);
}
exports.parse = parse;
/** The regular expression for testing if the property value is a circular object reference. */
const cycleMatcher = /^\$(?:\[(?:\d+|"(?:[^\\"\u0000-\u001f]|\\([\\"\/bfnrt]|u[0-9a-zA-Z]{4}))*")\])*$/;
function retrocycle($) {
    "use strict";
    function internalRetrocycle(obj) {
        if (obj && typeof obj === "object") {
            if (Array.isArray(obj)) {
                //  process each element
                obj.forEach(function (element, index) {
                    if (typeof element === "object" && element !== null) {
                        let path = element.$ref;
                        if (typeof path === "string" && cycleMatcher.test(path)) {
                            obj[index] = eval(path);
                        }
                        else {
                            internalRetrocycle(element);
                        }
                    }
                });
            }
            else {
                Object.keys(obj).forEach(function (name) {
                    let prop = obj[name];
                    if (typeof prop === "object" && prop !== null) {
                        let path = prop.$ref;
                        if (typeof path === "string" && cycleMatcher.test(path)) {
                            obj[name] = eval(path);
                        }
                        else {
                            internalRetrocycle(prop);
                        }
                    }
                });
            }
        }
    }
    internalRetrocycle($);
    return $;
}
exports.retrocycle = retrocycle;
/** Returns if `obj` is of an wrapper of simple type. */
function isPrimitiveWrapper(obj) {
    return (obj instanceof Boolean)
        || (obj instanceof Date)
        || (obj instanceof Number)
        || (obj instanceof RegExp)
        || (obj instanceof String);
}
//# sourceMappingURL=json-serialization-helper.js.map